package com.sun.xml.internal.ws.client;

import java.util.Iterator;
import java.util.logging.Level;
import java.util.ArrayList;
import javax.xml.ws.handler.Handler;
import java.util.List;
import javax.xml.ws.handler.PortInfo;
import javax.xml.ws.soap.SOAPBinding;
import com.sun.xml.internal.ws.util.HandlerAnnotationProcessor;
import java.util.HashMap;
import java.util.logging.Logger;
import com.sun.xml.internal.ws.util.HandlerAnnotationInfo;
import java.util.Map;
import com.sun.xml.internal.ws.handler.HandlerChainsModel;
import com.sun.istack.internal.Nullable;
import javax.xml.ws.handler.HandlerResolver;
import com.sun.xml.internal.ws.binding.BindingImpl;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.client.WSPortInfo;

abstract class HandlerConfigurator
{
    HandlerConfigurator() {
    }
    
    abstract void configureHandlers(@NotNull WSPortInfo p0, @NotNull BindingImpl p1);
    
    abstract HandlerResolver getResolver();
    
    static final class HandlerResolverImpl extends HandlerConfigurator
    {
        @Nullable
        private final HandlerResolver resolver;
        
        public HandlerResolverImpl(HandlerResolver resolver) {
            this.resolver = resolver;
        }
        
        @Override
        void configureHandlers(@NotNull WSPortInfo port, @NotNull BindingImpl binding) {
            if (this.resolver != null) {
                binding.setHandlerChain(this.resolver.getHandlerChain(port));
            }
        }
        
        @Override
        HandlerResolver getResolver() {
            return this.resolver;
        }
    }
    
    static final class AnnotationConfigurator extends HandlerConfigurator
    {
        private final HandlerChainsModel handlerModel;
        private final Map<WSPortInfo, HandlerAnnotationInfo> chainMap;
        private static final Logger logger;
        
        AnnotationConfigurator(WSServiceDelegate delegate) {
            this.chainMap = new HashMap();
            this.handlerModel = HandlerAnnotationProcessor.buildHandlerChainsModel(delegate.getServiceClass());
            assert this.handlerModel != null;
            return;
        }
        
        @Override
        void configureHandlers(WSPortInfo port, BindingImpl binding) {
            HandlerAnnotationInfo chain = (HandlerAnnotationInfo)this.chainMap.get(port);
            if (chain == null) {
                this.logGetChain(port);
                chain = this.handlerModel.getHandlersForPortInfo(port);
                this.chainMap.put(port, chain);
            }
            if ((binding instanceof SOAPBinding)) {
                ((SOAPBinding)binding).setRoles(chain.getRoles());
            }
            this.logSetChain(port, chain);
            binding.setHandlerChain(chain.getHandlers());
        }
        
        @Override
        HandlerResolver getResolver() {
            return new HandlerResolver() {
                public List<Handler> getHandlerChain(PortInfo portInfo) {
                    return new ArrayList(HandlerConfigurator.AnnotationConfigurator.this.handlerModel.getHandlersForPortInfo(portInfo).getHandlers());
                }
            };
        }
        
        private void logSetChain(WSPortInfo info, HandlerAnnotationInfo chain) {
            HandlerConfigurator.AnnotationConfigurator.logger.finer("Setting chain of length " + chain.getHandlers().size() + " for port info");
            this.logPortInfo(info, Level.FINER);
        }
        
        private void logGetChain(WSPortInfo info) {
            HandlerConfigurator.AnnotationConfigurator.logger.fine("No handler chain found for port info:");
            this.logPortInfo(info, Level.FINE);
            HandlerConfigurator.AnnotationConfigurator.logger.fine("Existing handler chains:");
            if (this.chainMap.isEmpty()) {
                HandlerConfigurator.AnnotationConfigurator.logger.fine("none");
            }
            else {
                for (WSPortInfo key : this.chainMap.keySet()) {
                    HandlerConfigurator.AnnotationConfigurator.logger.fine(((HandlerAnnotationInfo)this.chainMap.get(key)).getHandlers().size() + " handlers for port info ");
                    this.logPortInfo(key, Level.FINE);
                }
            }
        }
        
        private void logPortInfo(WSPortInfo info, Level level) {
            HandlerConfigurator.AnnotationConfigurator.logger.log(level, "binding: " + info.getBindingID() + "\nservice: " + info.getServiceName() + "\nport: " + info.getPortName());
        }
        
        static {
            logger = Logger.getLogger("com.sun.xml.internal.ws.handler");
        }
    }
}
